#define _CFFI_

/* We try to define Py_LIMITED_API before including Python.h.

   Mess: we can only define it if Py_DEBUG, Py_TRACE_REFS and
   Py_REF_DEBUG are not defined.  This is a best-effort approximation:
   we can learn about Py_DEBUG from pyconfig.h, but it is unclear if
   the same works for the other two macros.  Py_DEBUG implies them,
   but not the other way around.

   The implementation is messy (issue #350): on Windows, with _MSC_VER,
   we have to define Py_LIMITED_API even before including pyconfig.h.
   In that case, we guess what pyconfig.h will do to the macros above,
   and check our guess after the #include.

   Note that on Windows, with CPython 3.x, you need >= 3.5 and virtualenv
   version >= 16.0.0.  With older versions of either, you don't get a
   copy of PYTHON3.DLL in the virtualenv.  We can't check the version of
   CPython *before* we even include pyconfig.h.  ffi.set_source() puts
   a ``#define _CFFI_NO_LIMITED_API'' at the start of this file if it is
   running on Windows < 3.5, as an attempt at fixing it, but that's
   arguably wrong because it may not be the target version of Python.
   Still better than nothing I guess.  As another workaround, you can
   remove the definition of Py_LIMITED_API here.

   See also 'py_limited_api' in cffi/setuptools_ext.py.
*/
#if !defined(_CFFI_USE_EMBEDDING) && !defined(Py_LIMITED_API)
#  ifdef _MSC_VER
#    if !defined(_DEBUG) && !defined(Py_DEBUG) && !defined(Py_TRACE_REFS) && !defined(Py_REF_DEBUG) && !defined(_CFFI_NO_LIMITED_API)
#      define Py_LIMITED_API
#    endif
#    include <pyconfig.h>
     /* sanity-check: Py_LIMITED_API will cause crashes if any of these
        are also defined.  Normally, the Python file PC/pyconfig.h does not
        cause any of these to be defined, with the exception that _DEBUG
        causes Py_DEBUG.  Double-check that. */
#    ifdef Py_LIMITED_API
#      if defined(Py_DEBUG)
#        error "pyconfig.h unexpectedly defines Py_DEBUG, but Py_LIMITED_API is set"
#      endif
#      if defined(Py_TRACE_REFS)
#        error "pyconfig.h unexpectedly defines Py_TRACE_REFS, but Py_LIMITED_API is set"
#      endif
#      if defined(Py_REF_DEBUG)
#        error "pyconfig.h unexpectedly defines Py_REF_DEBUG, but Py_LIMITED_API is set"
#      endif
#    endif
#  else
#    include <pyconfig.h>
#    if !defined(Py_DEBUG) && !defined(Py_TRACE_REFS) && !defined(Py_REF_DEBUG) && !defined(_CFFI_NO_LIMITED_API)
#      define Py_LIMITED_API
#    endif
#  endif
#endif

#include <Python.h>
#ifdef __cplusplus
extern "C" {
#endif
#include <stddef.h>

/* This part is from file 'cffi/parse_c_type.h'.  It is copied at the
   beginning of C sources generated by CFFI's ffi.set_source(). */

typedef void *_cffi_opcode_t;

#define _CFFI_OP(opcode, arg)   (_cffi_opcode_t)(opcode | (((uintptr_t)(arg)) << 8))
#define _CFFI_GETOP(cffi_opcode)    ((unsigned char)(uintptr_t)cffi_opcode)
#define _CFFI_GETARG(cffi_opcode)   (((intptr_t)cffi_opcode) >> 8)

#define _CFFI_OP_PRIMITIVE       1
#define _CFFI_OP_POINTER         3
#define _CFFI_OP_ARRAY           5
#define _CFFI_OP_OPEN_ARRAY      7
#define _CFFI_OP_STRUCT_UNION    9
#define _CFFI_OP_ENUM           11
#define _CFFI_OP_FUNCTION       13
#define _CFFI_OP_FUNCTION_END   15
#define _CFFI_OP_NOOP           17
#define _CFFI_OP_BITFIELD       19
#define _CFFI_OP_TYPENAME       21
#define _CFFI_OP_CPYTHON_BLTN_V 23   // varargs
#define _CFFI_OP_CPYTHON_BLTN_N 25   // noargs
#define _CFFI_OP_CPYTHON_BLTN_O 27   // O  (i.e. a single arg)
#define _CFFI_OP_CONSTANT       29
#define _CFFI_OP_CONSTANT_INT   31
#define _CFFI_OP_GLOBAL_VAR     33
#define _CFFI_OP_DLOPEN_FUNC    35
#define _CFFI_OP_DLOPEN_CONST   37
#define _CFFI_OP_GLOBAL_VAR_F   39
#define _CFFI_OP_EXTERN_PYTHON  41

#define _CFFI_PRIM_VOID          0
#define _CFFI_PRIM_BOOL          1
#define _CFFI_PRIM_CHAR          2
#define _CFFI_PRIM_SCHAR         3
#define _CFFI_PRIM_UCHAR         4
#define _CFFI_PRIM_SHORT         5
#define _CFFI_PRIM_USHORT        6
#define _CFFI_PRIM_INT           7
#define _CFFI_PRIM_UINT          8
#define _CFFI_PRIM_LONG          9
#define _CFFI_PRIM_ULONG        10
#define _CFFI_PRIM_LONGLONG     11
#define _CFFI_PRIM_ULONGLONG    12
#define _CFFI_PRIM_FLOAT        13
#define _CFFI_PRIM_DOUBLE       14
#define _CFFI_PRIM_LONGDOUBLE   15

#define _CFFI_PRIM_WCHAR        16
#define _CFFI_PRIM_INT8         17
#define _CFFI_PRIM_UINT8        18
#define _CFFI_PRIM_INT16        19
#define _CFFI_PRIM_UINT16       20
#define _CFFI_PRIM_INT32        21
#define _CFFI_PRIM_UINT32       22
#define _CFFI_PRIM_INT64        23
#define _CFFI_PRIM_UINT64       24
#define _CFFI_PRIM_INTPTR       25
#define _CFFI_PRIM_UINTPTR      26
#define _CFFI_PRIM_PTRDIFF      27
#define _CFFI_PRIM_SIZE         28
#define _CFFI_PRIM_SSIZE        29
#define _CFFI_PRIM_INT_LEAST8   30
#define _CFFI_PRIM_UINT_LEAST8  31
#define _CFFI_PRIM_INT_LEAST16  32
#define _CFFI_PRIM_UINT_LEAST16 33
#define _CFFI_PRIM_INT_LEAST32  34
#define _CFFI_PRIM_UINT_LEAST32 35
#define _CFFI_PRIM_INT_LEAST64  36
#define _CFFI_PRIM_UINT_LEAST64 37
#define _CFFI_PRIM_INT_FAST8    38
#define _CFFI_PRIM_UINT_FAST8   39
#define _CFFI_PRIM_INT_FAST16   40
#define _CFFI_PRIM_UINT_FAST16  41
#define _CFFI_PRIM_INT_FAST32   42
#define _CFFI_PRIM_UINT_FAST32  43
#define _CFFI_PRIM_INT_FAST64   44
#define _CFFI_PRIM_UINT_FAST64  45
#define _CFFI_PRIM_INTMAX       46
#define _CFFI_PRIM_UINTMAX      47
#define _CFFI_PRIM_FLOATCOMPLEX 48
#define _CFFI_PRIM_DOUBLECOMPLEX 49
#define _CFFI_PRIM_CHAR16       50
#define _CFFI_PRIM_CHAR32       51

#define _CFFI__NUM_PRIM         52
#define _CFFI__UNKNOWN_PRIM           (-1)
#define _CFFI__UNKNOWN_FLOAT_PRIM     (-2)
#define _CFFI__UNKNOWN_LONG_DOUBLE    (-3)

#define _CFFI__IO_FILE_STRUCT         (-1)


struct _cffi_global_s {
    const char *name;
    void *address;
    _cffi_opcode_t type_op;
    void *size_or_direct_fn;  // OP_GLOBAL_VAR: size, or 0 if unknown
                              // OP_CPYTHON_BLTN_*: addr of direct function
};

struct _cffi_getconst_s {
    unsigned long long value;
    const struct _cffi_type_context_s *ctx;
    int gindex;
};

struct _cffi_struct_union_s {
    const char *name;
    int type_index;          // -> _cffi_types, on a OP_STRUCT_UNION
    int flags;               // _CFFI_F_* flags below
    size_t size;
    int alignment;
    int first_field_index;   // -> _cffi_fields array
    int num_fields;
};
#define _CFFI_F_UNION         0x01   // is a union, not a struct
#define _CFFI_F_CHECK_FIELDS  0x02   // complain if fields are not in the
                                     // "standard layout" or if some are missing
#define _CFFI_F_PACKED        0x04   // for CHECK_FIELDS, assume a packed struct
#define _CFFI_F_EXTERNAL      0x08   // in some other ffi.include()
#define _CFFI_F_OPAQUE        0x10   // opaque

struct _cffi_field_s {
    const char *name;
    size_t field_offset;
    size_t field_size;
    _cffi_opcode_t field_type_op;
};

struct _cffi_enum_s {
    const char *name;
    int type_index;          // -> _cffi_types, on a OP_ENUM
    int type_prim;           // _CFFI_PRIM_xxx
    const char *enumerators; // comma-delimited string
};

struct _cffi_typename_s {
    const char *name;
    int type_index;   /* if opaque, points to a possibly artificial
                         OP_STRUCT which is itself opaque */
};

struct _cffi_type_context_s {
    _cffi_opcode_t *types;
    const struct _cffi_global_s *globals;
    const struct _cffi_field_s *fields;
    const struct _cffi_struct_union_s *struct_unions;
    const struct _cffi_enum_s *enums;
    const struct _cffi_typename_s *typenames;
    int num_globals;
    int num_struct_unions;
    int num_enums;
    int num_typenames;
    const char *const *includes;
    int num_types;
    int flags;      /* future extension */
};

struct _cffi_parse_info_s {
    const struct _cffi_type_context_s *ctx;
    _cffi_opcode_t *output;
    unsigned int output_size;
    size_t error_location;
    const char *error_message;
};

struct _cffi_externpy_s {
    const char *name;
    size_t size_of_result;
    void *reserved1, *reserved2;
};

#ifdef _CFFI_INTERNAL
static int parse_c_type(struct _cffi_parse_info_s *info, const char *input);
static int search_in_globals(const struct _cffi_type_context_s *ctx,
                             const char *search, size_t search_len);
static int search_in_struct_unions(const struct _cffi_type_context_s *ctx,
                                   const char *search, size_t search_len);
#endif

/* this block of #ifs should be kept exactly identical between
   c/_cffi_backend.c, cffi/vengine_cpy.py, cffi/vengine_gen.py
   and cffi/_cffi_include.h */
#if defined(_MSC_VER)
# include <malloc.h>   /* for alloca() */
# if _MSC_VER < 1600   /* MSVC < 2010 */
   typedef __int8 int8_t;
   typedef __int16 int16_t;
   typedef __int32 int32_t;
   typedef __int64 int64_t;
   typedef unsigned __int8 uint8_t;
   typedef unsigned __int16 uint16_t;
   typedef unsigned __int32 uint32_t;
   typedef unsigned __int64 uint64_t;
   typedef __int8 int_least8_t;
   typedef __int16 int_least16_t;
   typedef __int32 int_least32_t;
   typedef __int64 int_least64_t;
   typedef unsigned __int8 uint_least8_t;
   typedef unsigned __int16 uint_least16_t;
   typedef unsigned __int32 uint_least32_t;
   typedef unsigned __int64 uint_least64_t;
   typedef __int8 int_fast8_t;
   typedef __int16 int_fast16_t;
   typedef __int32 int_fast32_t;
   typedef __int64 int_fast64_t;
   typedef unsigned __int8 uint_fast8_t;
   typedef unsigned __int16 uint_fast16_t;
   typedef unsigned __int32 uint_fast32_t;
   typedef unsigned __int64 uint_fast64_t;
   typedef __int64 intmax_t;
   typedef unsigned __int64 uintmax_t;
# else
#  include <stdint.h>
# endif
# if _MSC_VER < 1800   /* MSVC < 2013 */
#  ifndef __cplusplus
    typedef unsigned char _Bool;
#  endif
# endif
#else
# include <stdint.h>
# if (defined (__SVR4) && defined (__sun)) || defined(_AIX) || defined(__hpux)
#  include <alloca.h>
# endif
#endif

#ifdef __GNUC__
# define _CFFI_UNUSED_FN  __attribute__((unused))
#else
# define _CFFI_UNUSED_FN  /* nothing */
#endif

#ifdef __cplusplus
# ifndef _Bool
   typedef bool _Bool;   /* semi-hackish: C++ has no _Bool; bool is builtin */
# endif
#endif

/**********  CPython-specific section  **********/
#ifndef PYPY_VERSION


#if PY_MAJOR_VERSION >= 3
# define PyInt_FromLong PyLong_FromLong
#endif

#define _cffi_from_c_double PyFloat_FromDouble
#define _cffi_from_c_float PyFloat_FromDouble
#define _cffi_from_c_long PyInt_FromLong
#define _cffi_from_c_ulong PyLong_FromUnsignedLong
#define _cffi_from_c_longlong PyLong_FromLongLong
#define _cffi_from_c_ulonglong PyLong_FromUnsignedLongLong
#define _cffi_from_c__Bool PyBool_FromLong

#define _cffi_to_c_double PyFloat_AsDouble
#define _cffi_to_c_float PyFloat_AsDouble

#define _cffi_from_c_int(x, type)                                        \
    (((type)-1) > 0 ? /* unsigned */                                     \
        (sizeof(type) < sizeof(long) ?                                   \
            PyInt_FromLong((long)x) :                                    \
         sizeof(type) == sizeof(long) ?                                  \
            PyLong_FromUnsignedLong((unsigned long)x) :                  \
            PyLong_FromUnsignedLongLong((unsigned long long)x)) :        \
        (sizeof(type) <= sizeof(long) ?                                  \
            PyInt_FromLong((long)x) :                                    \
            PyLong_FromLongLong((long long)x)))

#define _cffi_to_c_int(o, type)                                          \
    ((type)(                                                             \
     sizeof(type) == 1 ? (((type)-1) > 0 ? (type)_cffi_to_c_u8(o)        \
                                         : (type)_cffi_to_c_i8(o)) :     \
     sizeof(type) == 2 ? (((type)-1) > 0 ? (type)_cffi_to_c_u16(o)       \
                                         : (type)_cffi_to_c_i16(o)) :    \
     sizeof(type) == 4 ? (((type)-1) > 0 ? (type)_cffi_to_c_u32(o)       \
                                         : (type)_cffi_to_c_i32(o)) :    \
     sizeof(type) == 8 ? (((type)-1) > 0 ? (type)_cffi_to_c_u64(o)       \
                                         : (type)_cffi_to_c_i64(o)) :    \
     (Py_FatalError("unsupported size for type " #type), (type)0)))

#define _cffi_to_c_i8                                                    \
                 ((int(*)(PyObject *))_cffi_exports[1])
#define _cffi_to_c_u8                                                    \
                 ((int(*)(PyObject *))_cffi_exports[2])
#define _cffi_to_c_i16                                                   \
                 ((int(*)(PyObject *))_cffi_exports[3])
#define _cffi_to_c_u16                                                   \
                 ((int(*)(PyObject *))_cffi_exports[4])
#define _cffi_to_c_i32                                                   \
                 ((int(*)(PyObject *))_cffi_exports[5])
#define _cffi_to_c_u32                                                   \
                 ((unsigned int(*)(PyObject *))_cffi_exports[6])
#define _cffi_to_c_i64                                                   \
                 ((long long(*)(PyObject *))_cffi_exports[7])
#define _cffi_to_c_u64                                                   \
                 ((unsigned long long(*)(PyObject *))_cffi_exports[8])
#define _cffi_to_c_char                                                  \
                 ((int(*)(PyObject *))_cffi_exports[9])
#define _cffi_from_c_pointer                                             \
    ((PyObject *(*)(char *, struct _cffi_ctypedescr *))_cffi_exports[10])
#define _cffi_to_c_pointer                                               \
    ((char *(*)(PyObject *, struct _cffi_ctypedescr *))_cffi_exports[11])
#define _cffi_get_struct_layout                                          \
    not used any more
#define _cffi_restore_errno                                              \
    ((void(*)(void))_cffi_exports[13])
#define _cffi_save_errno                                                 \
    ((void(*)(void))_cffi_exports[14])
#define _cffi_from_c_char                                                \
    ((PyObject *(*)(char))_cffi_exports[15])
#define _cffi_from_c_deref                                               \
    ((PyObject *(*)(char *, struct _cffi_ctypedescr *))_cffi_exports[16])
#define _cffi_to_c                                                       \
    ((int(*)(char *, struct _cffi_ctypedescr *, PyObject *))_cffi_exports[17])
#define _cffi_from_c_struct                                              \
    ((PyObject *(*)(char *, struct _cffi_ctypedescr *))_cffi_exports[18])
#define _cffi_to_c_wchar_t                                               \
    ((_cffi_wchar_t(*)(PyObject *))_cffi_exports[19])
#define _cffi_from_c_wchar_t                                             \
    ((PyObject *(*)(_cffi_wchar_t))_cffi_exports[20])
#define _cffi_to_c_long_double                                           \
    ((long double(*)(PyObject *))_cffi_exports[21])
#define _cffi_to_c__Bool                                                 \
    ((_Bool(*)(PyObject *))_cffi_exports[22])
#define _cffi_prepare_pointer_call_argument                              \
    ((Py_ssize_t(*)(struct _cffi_ctypedescr *,                           \
                    PyObject *, char **))_cffi_exports[23])
#define _cffi_convert_array_from_object                                  \
    ((int(*)(char *, struct _cffi_ctypedescr *, PyObject *))_cffi_exports[24])
#define _CFFI_CPIDX  25
#define _cffi_call_python                                                \
    ((void(*)(struct _cffi_externpy_s *, char *))_cffi_exports[_CFFI_CPIDX])
#define _cffi_to_c_wchar3216_t                                           \
    ((int(*)(PyObject *))_cffi_exports[26])
#define _cffi_from_c_wchar3216_t                                         \
    ((PyObject *(*)(int))_cffi_exports[27])
#define _CFFI_NUM_EXPORTS 28

struct _cffi_ctypedescr;

static void *_cffi_exports[_CFFI_NUM_EXPORTS];

#define _cffi_type(index)   (                           \
    assert((((uintptr_t)_cffi_types[index]) & 1) == 0), \
    (struct _cffi_ctypedescr *)_cffi_types[index])

static PyObject *_cffi_init(const char *module_name, Py_ssize_t version,
                            const struct _cffi_type_context_s *ctx)
{
    PyObject *module, *o_arg, *new_module;
    void *raw[] = {
        (void *)module_name,
        (void *)version,
        (void *)_cffi_exports,
        (void *)ctx,
    };

    module = PyImport_ImportModule("_cffi_backend");
    if (module == NULL)
        goto failure;

    o_arg = PyLong_FromVoidPtr((void *)raw);
    if (o_arg == NULL)
        goto failure;

    new_module = PyObject_CallMethod(
        module, (char *)"_init_cffi_1_0_external_module", (char *)"O", o_arg);

    Py_DECREF(o_arg);
    Py_DECREF(module);
    return new_module;

  failure:
    Py_XDECREF(module);
    return NULL;
}


#ifdef HAVE_WCHAR_H
typedef wchar_t _cffi_wchar_t;
#else
typedef uint16_t _cffi_wchar_t;   /* same random pick as _cffi_backend.c */
#endif

_CFFI_UNUSED_FN static uint16_t _cffi_to_c_char16_t(PyObject *o)
{
    if (sizeof(_cffi_wchar_t) == 2)
        return (uint16_t)_cffi_to_c_wchar_t(o);
    else
        return (uint16_t)_cffi_to_c_wchar3216_t(o);
}

_CFFI_UNUSED_FN static PyObject *_cffi_from_c_char16_t(uint16_t x)
{
    if (sizeof(_cffi_wchar_t) == 2)
        return _cffi_from_c_wchar_t((_cffi_wchar_t)x);
    else
        return _cffi_from_c_wchar3216_t((int)x);
}

_CFFI_UNUSED_FN static int _cffi_to_c_char32_t(PyObject *o)
{
    if (sizeof(_cffi_wchar_t) == 4)
        return (int)_cffi_to_c_wchar_t(o);
    else
        return (int)_cffi_to_c_wchar3216_t(o);
}

_CFFI_UNUSED_FN static PyObject *_cffi_from_c_char32_t(unsigned int x)
{
    if (sizeof(_cffi_wchar_t) == 4)
        return _cffi_from_c_wchar_t((_cffi_wchar_t)x);
    else
        return _cffi_from_c_wchar3216_t((int)x);
}

union _cffi_union_alignment_u {
    unsigned char m_char;
    unsigned short m_short;
    unsigned int m_int;
    unsigned long m_long;
    unsigned long long m_longlong;
    float m_float;
    double m_double;
    long double m_longdouble;
};

struct _cffi_freeme_s {
    struct _cffi_freeme_s *next;
    union _cffi_union_alignment_u alignment;
};

_CFFI_UNUSED_FN static int
_cffi_convert_array_argument(struct _cffi_ctypedescr *ctptr, PyObject *arg,
                             char **output_data, Py_ssize_t datasize,
                             struct _cffi_freeme_s **freeme)
{
    char *p;
    if (datasize < 0)
        return -1;

    p = *output_data;
    if (p == NULL) {
        struct _cffi_freeme_s *fp = (struct _cffi_freeme_s *)PyObject_Malloc(
            offsetof(struct _cffi_freeme_s, alignment) + (size_t)datasize);
        if (fp == NULL)
            return -1;
        fp->next = *freeme;
        *freeme = fp;
        p = *output_data = (char *)&fp->alignment;
    }
    memset((void *)p, 0, (size_t)datasize);
    return _cffi_convert_array_from_object(p, ctptr, arg);
}

_CFFI_UNUSED_FN static void
_cffi_free_array_arguments(struct _cffi_freeme_s *freeme)
{
    do {
        void *p = (void *)freeme;
        freeme = freeme->next;
        PyObject_Free(p);
    } while (freeme != NULL);
}

/**********  end CPython-specific section  **********/
#else
_CFFI_UNUSED_FN
static void (*_cffi_call_python_org)(struct _cffi_externpy_s *, char *);
# define _cffi_call_python  _cffi_call_python_org
#endif


#define _cffi_array_len(array)   (sizeof(array) / sizeof((array)[0]))

#define _cffi_prim_int(size, sign)                                      \
    ((size) == 1 ? ((sign) ? _CFFI_PRIM_INT8  : _CFFI_PRIM_UINT8)  :    \
     (size) == 2 ? ((sign) ? _CFFI_PRIM_INT16 : _CFFI_PRIM_UINT16) :    \
     (size) == 4 ? ((sign) ? _CFFI_PRIM_INT32 : _CFFI_PRIM_UINT32) :    \
     (size) == 8 ? ((sign) ? _CFFI_PRIM_INT64 : _CFFI_PRIM_UINT64) :    \
     _CFFI__UNKNOWN_PRIM)

#define _cffi_prim_float(size)                                          \
    ((size) == sizeof(float) ? _CFFI_PRIM_FLOAT :                       \
     (size) == sizeof(double) ? _CFFI_PRIM_DOUBLE :                     \
     (size) == sizeof(long double) ? _CFFI__UNKNOWN_LONG_DOUBLE :       \
     _CFFI__UNKNOWN_FLOAT_PRIM)

#define _cffi_check_int(got, got_nonpos, expected)      \
    ((got_nonpos) == (expected <= 0) &&                 \
     (got) == (unsigned long long)expected)

#ifdef MS_WIN32
# define _cffi_stdcall  __stdcall
#else
# define _cffi_stdcall  /* nothing */
#endif

#ifdef __cplusplus
}
#endif

/************************************************************/


#include <stdarg.h>
#include <xkbcommon/xkbregistry.h>

static void _log_handler(void *user_data,
                         enum rxkb_log_level level,
                         const char *message);

static void _log_handler_internal(
    struct rxkb_context *context,
    enum rxkb_log_level level,
    const char *format,
    va_list args)
{
  char buf[1024];
  void *user_data;

  user_data=rxkb_context_get_user_data(context);
  vsnprintf(buf, 1024, format, args);
  buf[1023]=0;
  _log_handler(user_data, level, buf);
}

void _set_log_handler_internal(struct rxkb_context *context)
{
  rxkb_context_set_log_fn(context, _log_handler_internal);
}



/************************************************************/

static void *_cffi_types[] = {
/*  0 */ _CFFI_OP(_CFFI_OP_FUNCTION, 118), // _Bool()(struct rxkb_context *)
/*  1 */ _CFFI_OP(_CFFI_OP_POINTER, 121), // struct rxkb_context *
/*  2 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/*  3 */ _CFFI_OP(_CFFI_OP_FUNCTION, 118), // _Bool()(struct rxkb_context *, char const *)
/*  4 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/*  5 */ _CFFI_OP(_CFFI_OP_POINTER, 119), // char const *
/*  6 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/*  7 */ _CFFI_OP(_CFFI_OP_FUNCTION, 118), // _Bool()(struct rxkb_option_group *)
/*  8 */ _CFFI_OP(_CFFI_OP_POINTER, 127), // struct rxkb_option_group *
/*  9 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 10 */ _CFFI_OP(_CFFI_OP_FUNCTION, 5), // char const *()(struct rxkb_iso3166_code *)
/* 11 */ _CFFI_OP(_CFFI_OP_POINTER, 122), // struct rxkb_iso3166_code *
/* 12 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 13 */ _CFFI_OP(_CFFI_OP_FUNCTION, 5), // char const *()(struct rxkb_iso639_code *)
/* 14 */ _CFFI_OP(_CFFI_OP_POINTER, 123), // struct rxkb_iso639_code *
/* 15 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 16 */ _CFFI_OP(_CFFI_OP_FUNCTION, 5), // char const *()(struct rxkb_layout *)
/* 17 */ _CFFI_OP(_CFFI_OP_POINTER, 124), // struct rxkb_layout *
/* 18 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 19 */ _CFFI_OP(_CFFI_OP_FUNCTION, 5), // char const *()(struct rxkb_model *)
/* 20 */ _CFFI_OP(_CFFI_OP_POINTER, 125), // struct rxkb_model *
/* 21 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 22 */ _CFFI_OP(_CFFI_OP_FUNCTION, 5), // char const *()(struct rxkb_option *)
/* 23 */ _CFFI_OP(_CFFI_OP_POINTER, 126), // struct rxkb_option *
/* 24 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 25 */ _CFFI_OP(_CFFI_OP_FUNCTION, 5), // char const *()(struct rxkb_option_group *)
/* 26 */ _CFFI_OP(_CFFI_OP_NOOP, 8),
/* 27 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 28 */ _CFFI_OP(_CFFI_OP_FUNCTION, 93), // enum rxkb_log_level()(struct rxkb_context *)
/* 29 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/* 30 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 31 */ _CFFI_OP(_CFFI_OP_FUNCTION, 120), // enum rxkb_popularity()(struct rxkb_layout *)
/* 32 */ _CFFI_OP(_CFFI_OP_NOOP, 17),
/* 33 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 34 */ _CFFI_OP(_CFFI_OP_FUNCTION, 120), // enum rxkb_popularity()(struct rxkb_model *)
/* 35 */ _CFFI_OP(_CFFI_OP_NOOP, 20),
/* 36 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 37 */ _CFFI_OP(_CFFI_OP_FUNCTION, 120), // enum rxkb_popularity()(struct rxkb_option *)
/* 38 */ _CFFI_OP(_CFFI_OP_NOOP, 23),
/* 39 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 40 */ _CFFI_OP(_CFFI_OP_FUNCTION, 120), // enum rxkb_popularity()(struct rxkb_option_group *)
/* 41 */ _CFFI_OP(_CFFI_OP_NOOP, 8),
/* 42 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 43 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1), // struct rxkb_context *()(enum rxkb_context_flags)
/* 44 */ _CFFI_OP(_CFFI_OP_ENUM, 0), // enum rxkb_context_flags
/* 45 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 46 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1), // struct rxkb_context *()(struct rxkb_context *)
/* 47 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/* 48 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 49 */ _CFFI_OP(_CFFI_OP_FUNCTION, 11), // struct rxkb_iso3166_code *()(struct rxkb_iso3166_code *)
/* 50 */ _CFFI_OP(_CFFI_OP_NOOP, 11),
/* 51 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 52 */ _CFFI_OP(_CFFI_OP_FUNCTION, 11), // struct rxkb_iso3166_code *()(struct rxkb_layout *)
/* 53 */ _CFFI_OP(_CFFI_OP_NOOP, 17),
/* 54 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 55 */ _CFFI_OP(_CFFI_OP_FUNCTION, 14), // struct rxkb_iso639_code *()(struct rxkb_iso639_code *)
/* 56 */ _CFFI_OP(_CFFI_OP_NOOP, 14),
/* 57 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 58 */ _CFFI_OP(_CFFI_OP_FUNCTION, 14), // struct rxkb_iso639_code *()(struct rxkb_layout *)
/* 59 */ _CFFI_OP(_CFFI_OP_NOOP, 17),
/* 60 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 61 */ _CFFI_OP(_CFFI_OP_FUNCTION, 17), // struct rxkb_layout *()(struct rxkb_context *)
/* 62 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/* 63 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 64 */ _CFFI_OP(_CFFI_OP_FUNCTION, 17), // struct rxkb_layout *()(struct rxkb_layout *)
/* 65 */ _CFFI_OP(_CFFI_OP_NOOP, 17),
/* 66 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 67 */ _CFFI_OP(_CFFI_OP_FUNCTION, 20), // struct rxkb_model *()(struct rxkb_context *)
/* 68 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/* 69 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 70 */ _CFFI_OP(_CFFI_OP_FUNCTION, 20), // struct rxkb_model *()(struct rxkb_model *)
/* 71 */ _CFFI_OP(_CFFI_OP_NOOP, 20),
/* 72 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 73 */ _CFFI_OP(_CFFI_OP_FUNCTION, 23), // struct rxkb_option *()(struct rxkb_option *)
/* 74 */ _CFFI_OP(_CFFI_OP_NOOP, 23),
/* 75 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 76 */ _CFFI_OP(_CFFI_OP_FUNCTION, 23), // struct rxkb_option *()(struct rxkb_option_group *)
/* 77 */ _CFFI_OP(_CFFI_OP_NOOP, 8),
/* 78 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 79 */ _CFFI_OP(_CFFI_OP_FUNCTION, 8), // struct rxkb_option_group *()(struct rxkb_context *)
/* 80 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/* 81 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 82 */ _CFFI_OP(_CFFI_OP_FUNCTION, 8), // struct rxkb_option_group *()(struct rxkb_option_group *)
/* 83 */ _CFFI_OP(_CFFI_OP_NOOP, 8),
/* 84 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 85 */ _CFFI_OP(_CFFI_OP_FUNCTION, 103), // void *()(struct rxkb_context *)
/* 86 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/* 87 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 88 */ _CFFI_OP(_CFFI_OP_FUNCTION, 129), // void()(struct rxkb_context *)
/* 89 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/* 90 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 91 */ _CFFI_OP(_CFFI_OP_FUNCTION, 129), // void()(struct rxkb_context *, enum rxkb_log_level)
/* 92 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/* 93 */ _CFFI_OP(_CFFI_OP_ENUM, 1), // enum rxkb_log_level
/* 94 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 95 */ _CFFI_OP(_CFFI_OP_FUNCTION, 129), // void()(struct rxkb_context *, enum rxkb_log_level, char const *, va_list)
/* 96 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/* 97 */ _CFFI_OP(_CFFI_OP_NOOP, 93),
/* 98 */ _CFFI_OP(_CFFI_OP_NOOP, 5),
/* 99 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 1), // va_list
/* 100 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 101 */ _CFFI_OP(_CFFI_OP_FUNCTION, 129), // void()(struct rxkb_context *, void *)
/* 102 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/* 103 */ _CFFI_OP(_CFFI_OP_POINTER, 129), // void *
/* 104 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 105 */ _CFFI_OP(_CFFI_OP_FUNCTION, 129), // void()(struct rxkb_context *, void(*)(struct rxkb_context *, enum rxkb_log_level, char const *, va_list))
/* 106 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/* 107 */ _CFFI_OP(_CFFI_OP_POINTER, 95), // void(*)(struct rxkb_context *, enum rxkb_log_level, char const *, va_list)
/* 108 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 109 */ _CFFI_OP(_CFFI_OP_FUNCTION, 129), // void()(void *)
/* 110 */ _CFFI_OP(_CFFI_OP_NOOP, 103),
/* 111 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 112 */ _CFFI_OP(_CFFI_OP_FUNCTION, 129), // void()(void *, enum rxkb_log_level, char const *)
/* 113 */ _CFFI_OP(_CFFI_OP_NOOP, 103),
/* 114 */ _CFFI_OP(_CFFI_OP_NOOP, 93),
/* 115 */ _CFFI_OP(_CFFI_OP_NOOP, 5),
/* 116 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 117 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 0), // FILE
/* 118 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 1), // _Bool
/* 119 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 2), // char
/* 120 */ _CFFI_OP(_CFFI_OP_ENUM, 2), // enum rxkb_popularity
/* 121 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 2), // struct rxkb_context
/* 122 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 3), // struct rxkb_iso3166_code
/* 123 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 4), // struct rxkb_iso639_code
/* 124 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 5), // struct rxkb_layout
/* 125 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 6), // struct rxkb_model
/* 126 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 7), // struct rxkb_option
/* 127 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 8), // struct rxkb_option_group
/* 128 */ _CFFI_OP(_CFFI_OP_POINTER, 112), // void(*)(void *, enum rxkb_log_level, char const *)
/* 129 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 0), // void
};

static int _cffi_const_RXKB_CONTEXT_NO_FLAGS(unsigned long long *o)
{
  int n = (RXKB_CONTEXT_NO_FLAGS) <= 0;
  *o = (unsigned long long)((RXKB_CONTEXT_NO_FLAGS) | 0);  /* check that RXKB_CONTEXT_NO_FLAGS is an integer */
  return n;
}

static int _cffi_const_RXKB_CONTEXT_NO_DEFAULT_INCLUDES(unsigned long long *o)
{
  int n = (RXKB_CONTEXT_NO_DEFAULT_INCLUDES) <= 0;
  *o = (unsigned long long)((RXKB_CONTEXT_NO_DEFAULT_INCLUDES) | 0);  /* check that RXKB_CONTEXT_NO_DEFAULT_INCLUDES is an integer */
  return n;
}

static int _cffi_const_RXKB_CONTEXT_LOAD_EXOTIC_RULES(unsigned long long *o)
{
  int n = (RXKB_CONTEXT_LOAD_EXOTIC_RULES) <= 0;
  *o = (unsigned long long)((RXKB_CONTEXT_LOAD_EXOTIC_RULES) | 0);  /* check that RXKB_CONTEXT_LOAD_EXOTIC_RULES is an integer */
  return n;
}

static int _cffi_const_RXKB_LOG_LEVEL_CRITICAL(unsigned long long *o)
{
  int n = (RXKB_LOG_LEVEL_CRITICAL) <= 0;
  *o = (unsigned long long)((RXKB_LOG_LEVEL_CRITICAL) | 0);  /* check that RXKB_LOG_LEVEL_CRITICAL is an integer */
  return n;
}

static int _cffi_const_RXKB_LOG_LEVEL_ERROR(unsigned long long *o)
{
  int n = (RXKB_LOG_LEVEL_ERROR) <= 0;
  *o = (unsigned long long)((RXKB_LOG_LEVEL_ERROR) | 0);  /* check that RXKB_LOG_LEVEL_ERROR is an integer */
  return n;
}

static int _cffi_const_RXKB_LOG_LEVEL_WARNING(unsigned long long *o)
{
  int n = (RXKB_LOG_LEVEL_WARNING) <= 0;
  *o = (unsigned long long)((RXKB_LOG_LEVEL_WARNING) | 0);  /* check that RXKB_LOG_LEVEL_WARNING is an integer */
  return n;
}

static int _cffi_const_RXKB_LOG_LEVEL_INFO(unsigned long long *o)
{
  int n = (RXKB_LOG_LEVEL_INFO) <= 0;
  *o = (unsigned long long)((RXKB_LOG_LEVEL_INFO) | 0);  /* check that RXKB_LOG_LEVEL_INFO is an integer */
  return n;
}

static int _cffi_const_RXKB_LOG_LEVEL_DEBUG(unsigned long long *o)
{
  int n = (RXKB_LOG_LEVEL_DEBUG) <= 0;
  *o = (unsigned long long)((RXKB_LOG_LEVEL_DEBUG) | 0);  /* check that RXKB_LOG_LEVEL_DEBUG is an integer */
  return n;
}

static int _cffi_const_RXKB_POPULARITY_STANDARD(unsigned long long *o)
{
  int n = (RXKB_POPULARITY_STANDARD) <= 0;
  *o = (unsigned long long)((RXKB_POPULARITY_STANDARD) | 0);  /* check that RXKB_POPULARITY_STANDARD is an integer */
  return n;
}

static int _cffi_const_RXKB_POPULARITY_EXOTIC(unsigned long long *o)
{
  int n = (RXKB_POPULARITY_EXOTIC) <= 0;
  *o = (unsigned long long)((RXKB_POPULARITY_EXOTIC) | 0);  /* check that RXKB_POPULARITY_EXOTIC is an integer */
  return n;
}

static struct _cffi_externpy_s _cffi_externpy___log_handler =
  { "xkbregistry._ffi._log_handler", 0, 0, 0 };

static void _log_handler(void * a0, enum rxkb_log_level a1, char const * a2)
{
  char a[24];
  char *p = a;
  *(void * *)(p + 0) = a0;
  *(enum rxkb_log_level *)(p + 8) = a1;
  *(char const * *)(p + 16) = a2;
  _cffi_call_python(&_cffi_externpy___log_handler, p);
}

static void _cffi_d__set_log_handler_internal(struct rxkb_context * x0)
{
  _set_log_handler_internal(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f__set_log_handler_internal(PyObject *self, PyObject *arg0)
{
  struct rxkb_context * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct rxkb_context *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { _set_log_handler_internal(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f__set_log_handler_internal _cffi_d__set_log_handler_internal
#endif

static void _cffi_d_free(void * x0)
{
  free(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_free(PyObject *self, PyObject *arg0)
{
  void * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(103), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (void *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(103), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { free(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_free _cffi_d_free
#endif

static enum rxkb_log_level _cffi_d_rxkb_context_get_log_level(struct rxkb_context * x0)
{
  return rxkb_context_get_log_level(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_rxkb_context_get_log_level(PyObject *self, PyObject *arg0)
{
  struct rxkb_context * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  enum rxkb_log_level result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct rxkb_context *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = rxkb_context_get_log_level(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_deref((char *)&result, _cffi_type(93));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_rxkb_context_get_log_level _cffi_d_rxkb_context_get_log_level
#endif

static void * _cffi_d_rxkb_context_get_user_data(struct rxkb_context * x0)
{
  return rxkb_context_get_user_data(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_rxkb_context_get_user_data(PyObject *self, PyObject *arg0)
{
  struct rxkb_context * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  void * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct rxkb_context *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = rxkb_context_get_user_data(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(103));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_rxkb_context_get_user_data _cffi_d_rxkb_context_get_user_data
#endif

static _Bool _cffi_d_rxkb_context_include_path_append(struct rxkb_context * x0, char const * x1)
{
  return rxkb_context_include_path_append(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_rxkb_context_include_path_append(PyObject *self, PyObject *args)
{
  struct rxkb_context * x0;
  char const * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  _Bool result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "rxkb_context_include_path_append", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct rxkb_context *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = rxkb_context_include_path_append(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c__Bool(result);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_rxkb_context_include_path_append _cffi_d_rxkb_context_include_path_append
#endif

static _Bool _cffi_d_rxkb_context_include_path_append_default(struct rxkb_context * x0)
{
  return rxkb_context_include_path_append_default(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_rxkb_context_include_path_append_default(PyObject *self, PyObject *arg0)
{
  struct rxkb_context * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  _Bool result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct rxkb_context *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = rxkb_context_include_path_append_default(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c__Bool(result);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_rxkb_context_include_path_append_default _cffi_d_rxkb_context_include_path_append_default
#endif

static struct rxkb_context * _cffi_d_rxkb_context_new(enum rxkb_context_flags x0)
{
  return rxkb_context_new(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_rxkb_context_new(PyObject *self, PyObject *arg0)
{
  enum rxkb_context_flags x0;
  struct rxkb_context * result;
  PyObject *pyresult;

  if (_cffi_to_c((char *)&x0, _cffi_type(44), arg0) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = rxkb_context_new(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(1));
  return pyresult;
}
#else
#  define _cffi_f_rxkb_context_new _cffi_d_rxkb_context_new
#endif

static _Bool _cffi_d_rxkb_context_parse(struct rxkb_context * x0, char const * x1)
{
  return rxkb_context_parse(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_rxkb_context_parse(PyObject *self, PyObject *args)
{
  struct rxkb_context * x0;
  char const * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  _Bool result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "rxkb_context_parse", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct rxkb_context *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = rxkb_context_parse(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c__Bool(result);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_rxkb_context_parse _cffi_d_rxkb_context_parse
#endif

static _Bool _cffi_d_rxkb_context_parse_default_ruleset(struct rxkb_context * x0)
{
  return rxkb_context_parse_default_ruleset(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_rxkb_context_parse_default_ruleset(PyObject *self, PyObject *arg0)
{
  struct rxkb_context * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  _Bool result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct rxkb_context *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = rxkb_context_parse_default_ruleset(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c__Bool(result);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_rxkb_context_parse_default_ruleset _cffi_d_rxkb_context_parse_default_ruleset
#endif

static struct rxkb_context * _cffi_d_rxkb_context_ref(struct rxkb_context * x0)
{
  return rxkb_context_ref(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_rxkb_context_ref(PyObject *self, PyObject *arg0)
{
  struct rxkb_context * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  struct rxkb_context * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct rxkb_context *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = rxkb_context_ref(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(1));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_rxkb_context_ref _cffi_d_rxkb_context_ref
#endif

static void _cffi_d_rxkb_context_set_log_fn(struct rxkb_context * x0, void(* x1)(struct rxkb_context *, enum rxkb_log_level, char const *, va_list))
{
  rxkb_context_set_log_fn(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_rxkb_context_set_log_fn(PyObject *self, PyObject *args)
{
  struct rxkb_context * x0;
  void(* x1)(struct rxkb_context *, enum rxkb_log_level, char const *, va_list);
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "rxkb_context_set_log_fn", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct rxkb_context *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = (void(*)(struct rxkb_context *, enum rxkb_log_level, char const *, va_list))_cffi_to_c_pointer(arg1, _cffi_type(107));
  if (x1 == (void(*)(struct rxkb_context *, enum rxkb_log_level, char const *, va_list))NULL && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { rxkb_context_set_log_fn(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_rxkb_context_set_log_fn _cffi_d_rxkb_context_set_log_fn
#endif

static void _cffi_d_rxkb_context_set_log_level(struct rxkb_context * x0, enum rxkb_log_level x1)
{
  rxkb_context_set_log_level(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_rxkb_context_set_log_level(PyObject *self, PyObject *args)
{
  struct rxkb_context * x0;
  enum rxkb_log_level x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "rxkb_context_set_log_level", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct rxkb_context *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  if (_cffi_to_c((char *)&x1, _cffi_type(93), arg1) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { rxkb_context_set_log_level(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_rxkb_context_set_log_level _cffi_d_rxkb_context_set_log_level
#endif

static void _cffi_d_rxkb_context_set_user_data(struct rxkb_context * x0, void * x1)
{
  rxkb_context_set_user_data(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_rxkb_context_set_user_data(PyObject *self, PyObject *args)
{
  struct rxkb_context * x0;
  void * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "rxkb_context_set_user_data", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct rxkb_context *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(103), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (void *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(103), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { rxkb_context_set_user_data(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_rxkb_context_set_user_data _cffi_d_rxkb_context_set_user_data
#endif

static struct rxkb_context * _cffi_d_rxkb_context_unref(struct rxkb_context * x0)
{
  return rxkb_context_unref(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_rxkb_context_unref(PyObject *self, PyObject *arg0)
{
  struct rxkb_context * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  struct rxkb_context * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct rxkb_context *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = rxkb_context_unref(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(1));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_rxkb_context_unref _cffi_d_rxkb_context_unref
#endif

static char const * _cffi_d_rxkb_iso3166_code_get_code(struct rxkb_iso3166_code * x0)
{
  return rxkb_iso3166_code_get_code(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_rxkb_iso3166_code_get_code(PyObject *self, PyObject *arg0)
{
  struct rxkb_iso3166_code * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  char const * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(11), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct rxkb_iso3166_code *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(11), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = rxkb_iso3166_code_get_code(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(5));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_rxkb_iso3166_code_get_code _cffi_d_rxkb_iso3166_code_get_code
#endif

static struct rxkb_iso3166_code * _cffi_d_rxkb_iso3166_code_next(struct rxkb_iso3166_code * x0)
{
  return rxkb_iso3166_code_next(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_rxkb_iso3166_code_next(PyObject *self, PyObject *arg0)
{
  struct rxkb_iso3166_code * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  struct rxkb_iso3166_code * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(11), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct rxkb_iso3166_code *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(11), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = rxkb_iso3166_code_next(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(11));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_rxkb_iso3166_code_next _cffi_d_rxkb_iso3166_code_next
#endif

static struct rxkb_iso3166_code * _cffi_d_rxkb_iso3166_code_ref(struct rxkb_iso3166_code * x0)
{
  return rxkb_iso3166_code_ref(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_rxkb_iso3166_code_ref(PyObject *self, PyObject *arg0)
{
  struct rxkb_iso3166_code * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  struct rxkb_iso3166_code * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(11), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct rxkb_iso3166_code *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(11), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = rxkb_iso3166_code_ref(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(11));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_rxkb_iso3166_code_ref _cffi_d_rxkb_iso3166_code_ref
#endif

static struct rxkb_iso3166_code * _cffi_d_rxkb_iso3166_code_unref(struct rxkb_iso3166_code * x0)
{
  return rxkb_iso3166_code_unref(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_rxkb_iso3166_code_unref(PyObject *self, PyObject *arg0)
{
  struct rxkb_iso3166_code * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  struct rxkb_iso3166_code * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(11), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct rxkb_iso3166_code *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(11), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = rxkb_iso3166_code_unref(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(11));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_rxkb_iso3166_code_unref _cffi_d_rxkb_iso3166_code_unref
#endif

static char const * _cffi_d_rxkb_iso639_code_get_code(struct rxkb_iso639_code * x0)
{
  return rxkb_iso639_code_get_code(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_rxkb_iso639_code_get_code(PyObject *self, PyObject *arg0)
{
  struct rxkb_iso639_code * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  char const * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(14), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct rxkb_iso639_code *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(14), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = rxkb_iso639_code_get_code(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(5));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_rxkb_iso639_code_get_code _cffi_d_rxkb_iso639_code_get_code
#endif

static struct rxkb_iso639_code * _cffi_d_rxkb_iso639_code_next(struct rxkb_iso639_code * x0)
{
  return rxkb_iso639_code_next(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_rxkb_iso639_code_next(PyObject *self, PyObject *arg0)
{
  struct rxkb_iso639_code * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  struct rxkb_iso639_code * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(14), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct rxkb_iso639_code *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(14), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = rxkb_iso639_code_next(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(14));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_rxkb_iso639_code_next _cffi_d_rxkb_iso639_code_next
#endif

static struct rxkb_iso639_code * _cffi_d_rxkb_iso639_code_ref(struct rxkb_iso639_code * x0)
{
  return rxkb_iso639_code_ref(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_rxkb_iso639_code_ref(PyObject *self, PyObject *arg0)
{
  struct rxkb_iso639_code * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  struct rxkb_iso639_code * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(14), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct rxkb_iso639_code *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(14), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = rxkb_iso639_code_ref(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(14));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_rxkb_iso639_code_ref _cffi_d_rxkb_iso639_code_ref
#endif

static struct rxkb_iso639_code * _cffi_d_rxkb_iso639_code_unref(struct rxkb_iso639_code * x0)
{
  return rxkb_iso639_code_unref(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_rxkb_iso639_code_unref(PyObject *self, PyObject *arg0)
{
  struct rxkb_iso639_code * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  struct rxkb_iso639_code * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(14), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct rxkb_iso639_code *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(14), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = rxkb_iso639_code_unref(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(14));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_rxkb_iso639_code_unref _cffi_d_rxkb_iso639_code_unref
#endif

static struct rxkb_layout * _cffi_d_rxkb_layout_first(struct rxkb_context * x0)
{
  return rxkb_layout_first(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_rxkb_layout_first(PyObject *self, PyObject *arg0)
{
  struct rxkb_context * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  struct rxkb_layout * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct rxkb_context *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = rxkb_layout_first(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(17));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_rxkb_layout_first _cffi_d_rxkb_layout_first
#endif

static char const * _cffi_d_rxkb_layout_get_brief(struct rxkb_layout * x0)
{
  return rxkb_layout_get_brief(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_rxkb_layout_get_brief(PyObject *self, PyObject *arg0)
{
  struct rxkb_layout * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  char const * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(17), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct rxkb_layout *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(17), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = rxkb_layout_get_brief(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(5));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_rxkb_layout_get_brief _cffi_d_rxkb_layout_get_brief
#endif

static char const * _cffi_d_rxkb_layout_get_description(struct rxkb_layout * x0)
{
  return rxkb_layout_get_description(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_rxkb_layout_get_description(PyObject *self, PyObject *arg0)
{
  struct rxkb_layout * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  char const * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(17), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct rxkb_layout *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(17), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = rxkb_layout_get_description(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(5));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_rxkb_layout_get_description _cffi_d_rxkb_layout_get_description
#endif

static struct rxkb_iso3166_code * _cffi_d_rxkb_layout_get_iso3166_first(struct rxkb_layout * x0)
{
  return rxkb_layout_get_iso3166_first(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_rxkb_layout_get_iso3166_first(PyObject *self, PyObject *arg0)
{
  struct rxkb_layout * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  struct rxkb_iso3166_code * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(17), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct rxkb_layout *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(17), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = rxkb_layout_get_iso3166_first(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(11));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_rxkb_layout_get_iso3166_first _cffi_d_rxkb_layout_get_iso3166_first
#endif

static struct rxkb_iso639_code * _cffi_d_rxkb_layout_get_iso639_first(struct rxkb_layout * x0)
{
  return rxkb_layout_get_iso639_first(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_rxkb_layout_get_iso639_first(PyObject *self, PyObject *arg0)
{
  struct rxkb_layout * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  struct rxkb_iso639_code * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(17), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct rxkb_layout *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(17), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = rxkb_layout_get_iso639_first(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(14));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_rxkb_layout_get_iso639_first _cffi_d_rxkb_layout_get_iso639_first
#endif

static char const * _cffi_d_rxkb_layout_get_name(struct rxkb_layout * x0)
{
  return rxkb_layout_get_name(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_rxkb_layout_get_name(PyObject *self, PyObject *arg0)
{
  struct rxkb_layout * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  char const * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(17), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct rxkb_layout *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(17), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = rxkb_layout_get_name(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(5));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_rxkb_layout_get_name _cffi_d_rxkb_layout_get_name
#endif

static enum rxkb_popularity _cffi_d_rxkb_layout_get_popularity(struct rxkb_layout * x0)
{
  return rxkb_layout_get_popularity(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_rxkb_layout_get_popularity(PyObject *self, PyObject *arg0)
{
  struct rxkb_layout * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  enum rxkb_popularity result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(17), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct rxkb_layout *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(17), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = rxkb_layout_get_popularity(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_deref((char *)&result, _cffi_type(120));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_rxkb_layout_get_popularity _cffi_d_rxkb_layout_get_popularity
#endif

static char const * _cffi_d_rxkb_layout_get_variant(struct rxkb_layout * x0)
{
  return rxkb_layout_get_variant(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_rxkb_layout_get_variant(PyObject *self, PyObject *arg0)
{
  struct rxkb_layout * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  char const * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(17), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct rxkb_layout *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(17), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = rxkb_layout_get_variant(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(5));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_rxkb_layout_get_variant _cffi_d_rxkb_layout_get_variant
#endif

static struct rxkb_layout * _cffi_d_rxkb_layout_next(struct rxkb_layout * x0)
{
  return rxkb_layout_next(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_rxkb_layout_next(PyObject *self, PyObject *arg0)
{
  struct rxkb_layout * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  struct rxkb_layout * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(17), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct rxkb_layout *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(17), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = rxkb_layout_next(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(17));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_rxkb_layout_next _cffi_d_rxkb_layout_next
#endif

static struct rxkb_layout * _cffi_d_rxkb_layout_ref(struct rxkb_layout * x0)
{
  return rxkb_layout_ref(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_rxkb_layout_ref(PyObject *self, PyObject *arg0)
{
  struct rxkb_layout * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  struct rxkb_layout * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(17), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct rxkb_layout *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(17), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = rxkb_layout_ref(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(17));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_rxkb_layout_ref _cffi_d_rxkb_layout_ref
#endif

static struct rxkb_layout * _cffi_d_rxkb_layout_unref(struct rxkb_layout * x0)
{
  return rxkb_layout_unref(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_rxkb_layout_unref(PyObject *self, PyObject *arg0)
{
  struct rxkb_layout * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  struct rxkb_layout * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(17), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct rxkb_layout *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(17), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = rxkb_layout_unref(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(17));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_rxkb_layout_unref _cffi_d_rxkb_layout_unref
#endif

static struct rxkb_model * _cffi_d_rxkb_model_first(struct rxkb_context * x0)
{
  return rxkb_model_first(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_rxkb_model_first(PyObject *self, PyObject *arg0)
{
  struct rxkb_context * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  struct rxkb_model * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct rxkb_context *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = rxkb_model_first(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(20));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_rxkb_model_first _cffi_d_rxkb_model_first
#endif

static char const * _cffi_d_rxkb_model_get_description(struct rxkb_model * x0)
{
  return rxkb_model_get_description(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_rxkb_model_get_description(PyObject *self, PyObject *arg0)
{
  struct rxkb_model * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  char const * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(20), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct rxkb_model *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(20), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = rxkb_model_get_description(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(5));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_rxkb_model_get_description _cffi_d_rxkb_model_get_description
#endif

static char const * _cffi_d_rxkb_model_get_name(struct rxkb_model * x0)
{
  return rxkb_model_get_name(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_rxkb_model_get_name(PyObject *self, PyObject *arg0)
{
  struct rxkb_model * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  char const * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(20), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct rxkb_model *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(20), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = rxkb_model_get_name(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(5));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_rxkb_model_get_name _cffi_d_rxkb_model_get_name
#endif

static enum rxkb_popularity _cffi_d_rxkb_model_get_popularity(struct rxkb_model * x0)
{
  return rxkb_model_get_popularity(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_rxkb_model_get_popularity(PyObject *self, PyObject *arg0)
{
  struct rxkb_model * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  enum rxkb_popularity result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(20), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct rxkb_model *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(20), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = rxkb_model_get_popularity(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_deref((char *)&result, _cffi_type(120));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_rxkb_model_get_popularity _cffi_d_rxkb_model_get_popularity
#endif

static char const * _cffi_d_rxkb_model_get_vendor(struct rxkb_model * x0)
{
  return rxkb_model_get_vendor(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_rxkb_model_get_vendor(PyObject *self, PyObject *arg0)
{
  struct rxkb_model * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  char const * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(20), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct rxkb_model *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(20), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = rxkb_model_get_vendor(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(5));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_rxkb_model_get_vendor _cffi_d_rxkb_model_get_vendor
#endif

static struct rxkb_model * _cffi_d_rxkb_model_next(struct rxkb_model * x0)
{
  return rxkb_model_next(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_rxkb_model_next(PyObject *self, PyObject *arg0)
{
  struct rxkb_model * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  struct rxkb_model * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(20), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct rxkb_model *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(20), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = rxkb_model_next(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(20));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_rxkb_model_next _cffi_d_rxkb_model_next
#endif

static struct rxkb_model * _cffi_d_rxkb_model_ref(struct rxkb_model * x0)
{
  return rxkb_model_ref(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_rxkb_model_ref(PyObject *self, PyObject *arg0)
{
  struct rxkb_model * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  struct rxkb_model * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(20), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct rxkb_model *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(20), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = rxkb_model_ref(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(20));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_rxkb_model_ref _cffi_d_rxkb_model_ref
#endif

static struct rxkb_model * _cffi_d_rxkb_model_unref(struct rxkb_model * x0)
{
  return rxkb_model_unref(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_rxkb_model_unref(PyObject *self, PyObject *arg0)
{
  struct rxkb_model * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  struct rxkb_model * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(20), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct rxkb_model *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(20), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = rxkb_model_unref(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(20));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_rxkb_model_unref _cffi_d_rxkb_model_unref
#endif

static struct rxkb_option * _cffi_d_rxkb_option_first(struct rxkb_option_group * x0)
{
  return rxkb_option_first(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_rxkb_option_first(PyObject *self, PyObject *arg0)
{
  struct rxkb_option_group * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  struct rxkb_option * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(8), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct rxkb_option_group *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(8), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = rxkb_option_first(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(23));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_rxkb_option_first _cffi_d_rxkb_option_first
#endif

static char const * _cffi_d_rxkb_option_get_brief(struct rxkb_option * x0)
{
  return rxkb_option_get_brief(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_rxkb_option_get_brief(PyObject *self, PyObject *arg0)
{
  struct rxkb_option * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  char const * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(23), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct rxkb_option *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(23), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = rxkb_option_get_brief(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(5));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_rxkb_option_get_brief _cffi_d_rxkb_option_get_brief
#endif

static char const * _cffi_d_rxkb_option_get_description(struct rxkb_option * x0)
{
  return rxkb_option_get_description(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_rxkb_option_get_description(PyObject *self, PyObject *arg0)
{
  struct rxkb_option * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  char const * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(23), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct rxkb_option *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(23), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = rxkb_option_get_description(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(5));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_rxkb_option_get_description _cffi_d_rxkb_option_get_description
#endif

static char const * _cffi_d_rxkb_option_get_name(struct rxkb_option * x0)
{
  return rxkb_option_get_name(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_rxkb_option_get_name(PyObject *self, PyObject *arg0)
{
  struct rxkb_option * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  char const * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(23), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct rxkb_option *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(23), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = rxkb_option_get_name(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(5));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_rxkb_option_get_name _cffi_d_rxkb_option_get_name
#endif

static enum rxkb_popularity _cffi_d_rxkb_option_get_popularity(struct rxkb_option * x0)
{
  return rxkb_option_get_popularity(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_rxkb_option_get_popularity(PyObject *self, PyObject *arg0)
{
  struct rxkb_option * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  enum rxkb_popularity result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(23), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct rxkb_option *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(23), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = rxkb_option_get_popularity(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_deref((char *)&result, _cffi_type(120));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_rxkb_option_get_popularity _cffi_d_rxkb_option_get_popularity
#endif

static _Bool _cffi_d_rxkb_option_group_allows_multiple(struct rxkb_option_group * x0)
{
  return rxkb_option_group_allows_multiple(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_rxkb_option_group_allows_multiple(PyObject *self, PyObject *arg0)
{
  struct rxkb_option_group * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  _Bool result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(8), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct rxkb_option_group *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(8), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = rxkb_option_group_allows_multiple(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c__Bool(result);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_rxkb_option_group_allows_multiple _cffi_d_rxkb_option_group_allows_multiple
#endif

static struct rxkb_option_group * _cffi_d_rxkb_option_group_first(struct rxkb_context * x0)
{
  return rxkb_option_group_first(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_rxkb_option_group_first(PyObject *self, PyObject *arg0)
{
  struct rxkb_context * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  struct rxkb_option_group * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct rxkb_context *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = rxkb_option_group_first(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(8));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_rxkb_option_group_first _cffi_d_rxkb_option_group_first
#endif

static char const * _cffi_d_rxkb_option_group_get_description(struct rxkb_option_group * x0)
{
  return rxkb_option_group_get_description(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_rxkb_option_group_get_description(PyObject *self, PyObject *arg0)
{
  struct rxkb_option_group * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  char const * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(8), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct rxkb_option_group *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(8), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = rxkb_option_group_get_description(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(5));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_rxkb_option_group_get_description _cffi_d_rxkb_option_group_get_description
#endif

static char const * _cffi_d_rxkb_option_group_get_name(struct rxkb_option_group * x0)
{
  return rxkb_option_group_get_name(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_rxkb_option_group_get_name(PyObject *self, PyObject *arg0)
{
  struct rxkb_option_group * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  char const * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(8), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct rxkb_option_group *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(8), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = rxkb_option_group_get_name(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(5));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_rxkb_option_group_get_name _cffi_d_rxkb_option_group_get_name
#endif

static enum rxkb_popularity _cffi_d_rxkb_option_group_get_popularity(struct rxkb_option_group * x0)
{
  return rxkb_option_group_get_popularity(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_rxkb_option_group_get_popularity(PyObject *self, PyObject *arg0)
{
  struct rxkb_option_group * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  enum rxkb_popularity result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(8), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct rxkb_option_group *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(8), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = rxkb_option_group_get_popularity(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_deref((char *)&result, _cffi_type(120));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_rxkb_option_group_get_popularity _cffi_d_rxkb_option_group_get_popularity
#endif

static struct rxkb_option_group * _cffi_d_rxkb_option_group_next(struct rxkb_option_group * x0)
{
  return rxkb_option_group_next(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_rxkb_option_group_next(PyObject *self, PyObject *arg0)
{
  struct rxkb_option_group * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  struct rxkb_option_group * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(8), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct rxkb_option_group *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(8), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = rxkb_option_group_next(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(8));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_rxkb_option_group_next _cffi_d_rxkb_option_group_next
#endif

static struct rxkb_option_group * _cffi_d_rxkb_option_group_ref(struct rxkb_option_group * x0)
{
  return rxkb_option_group_ref(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_rxkb_option_group_ref(PyObject *self, PyObject *arg0)
{
  struct rxkb_option_group * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  struct rxkb_option_group * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(8), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct rxkb_option_group *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(8), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = rxkb_option_group_ref(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(8));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_rxkb_option_group_ref _cffi_d_rxkb_option_group_ref
#endif

static struct rxkb_option_group * _cffi_d_rxkb_option_group_unref(struct rxkb_option_group * x0)
{
  return rxkb_option_group_unref(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_rxkb_option_group_unref(PyObject *self, PyObject *arg0)
{
  struct rxkb_option_group * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  struct rxkb_option_group * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(8), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct rxkb_option_group *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(8), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = rxkb_option_group_unref(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(8));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_rxkb_option_group_unref _cffi_d_rxkb_option_group_unref
#endif

static struct rxkb_option * _cffi_d_rxkb_option_next(struct rxkb_option * x0)
{
  return rxkb_option_next(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_rxkb_option_next(PyObject *self, PyObject *arg0)
{
  struct rxkb_option * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  struct rxkb_option * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(23), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct rxkb_option *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(23), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = rxkb_option_next(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(23));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_rxkb_option_next _cffi_d_rxkb_option_next
#endif

static struct rxkb_option * _cffi_d_rxkb_option_ref(struct rxkb_option * x0)
{
  return rxkb_option_ref(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_rxkb_option_ref(PyObject *self, PyObject *arg0)
{
  struct rxkb_option * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  struct rxkb_option * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(23), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct rxkb_option *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(23), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = rxkb_option_ref(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(23));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_rxkb_option_ref _cffi_d_rxkb_option_ref
#endif

static struct rxkb_option * _cffi_d_rxkb_option_unref(struct rxkb_option * x0)
{
  return rxkb_option_unref(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_rxkb_option_unref(PyObject *self, PyObject *arg0)
{
  struct rxkb_option * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  struct rxkb_option * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(23), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct rxkb_option *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(23), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = rxkb_option_unref(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(23));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_rxkb_option_unref _cffi_d_rxkb_option_unref
#endif

static const struct _cffi_global_s _cffi_globals[] = {
  { "RXKB_CONTEXT_LOAD_EXOTIC_RULES", (void *)_cffi_const_RXKB_CONTEXT_LOAD_EXOTIC_RULES, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "RXKB_CONTEXT_NO_DEFAULT_INCLUDES", (void *)_cffi_const_RXKB_CONTEXT_NO_DEFAULT_INCLUDES, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "RXKB_CONTEXT_NO_FLAGS", (void *)_cffi_const_RXKB_CONTEXT_NO_FLAGS, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "RXKB_LOG_LEVEL_CRITICAL", (void *)_cffi_const_RXKB_LOG_LEVEL_CRITICAL, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "RXKB_LOG_LEVEL_DEBUG", (void *)_cffi_const_RXKB_LOG_LEVEL_DEBUG, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "RXKB_LOG_LEVEL_ERROR", (void *)_cffi_const_RXKB_LOG_LEVEL_ERROR, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "RXKB_LOG_LEVEL_INFO", (void *)_cffi_const_RXKB_LOG_LEVEL_INFO, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "RXKB_LOG_LEVEL_WARNING", (void *)_cffi_const_RXKB_LOG_LEVEL_WARNING, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "RXKB_POPULARITY_EXOTIC", (void *)_cffi_const_RXKB_POPULARITY_EXOTIC, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "RXKB_POPULARITY_STANDARD", (void *)_cffi_const_RXKB_POPULARITY_STANDARD, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "_log_handler", (void *)&_cffi_externpy___log_handler, _CFFI_OP(_CFFI_OP_EXTERN_PYTHON, 128), (void *)_log_handler },
  { "_set_log_handler_internal", (void *)_cffi_f__set_log_handler_internal, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 88), (void *)_cffi_d__set_log_handler_internal },
  { "free", (void *)_cffi_f_free, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 109), (void *)_cffi_d_free },
  { "rxkb_context_get_log_level", (void *)_cffi_f_rxkb_context_get_log_level, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 28), (void *)_cffi_d_rxkb_context_get_log_level },
  { "rxkb_context_get_user_data", (void *)_cffi_f_rxkb_context_get_user_data, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 85), (void *)_cffi_d_rxkb_context_get_user_data },
  { "rxkb_context_include_path_append", (void *)_cffi_f_rxkb_context_include_path_append, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 3), (void *)_cffi_d_rxkb_context_include_path_append },
  { "rxkb_context_include_path_append_default", (void *)_cffi_f_rxkb_context_include_path_append_default, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 0), (void *)_cffi_d_rxkb_context_include_path_append_default },
  { "rxkb_context_new", (void *)_cffi_f_rxkb_context_new, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 43), (void *)_cffi_d_rxkb_context_new },
  { "rxkb_context_parse", (void *)_cffi_f_rxkb_context_parse, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 3), (void *)_cffi_d_rxkb_context_parse },
  { "rxkb_context_parse_default_ruleset", (void *)_cffi_f_rxkb_context_parse_default_ruleset, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 0), (void *)_cffi_d_rxkb_context_parse_default_ruleset },
  { "rxkb_context_ref", (void *)_cffi_f_rxkb_context_ref, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 46), (void *)_cffi_d_rxkb_context_ref },
  { "rxkb_context_set_log_fn", (void *)_cffi_f_rxkb_context_set_log_fn, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 105), (void *)_cffi_d_rxkb_context_set_log_fn },
  { "rxkb_context_set_log_level", (void *)_cffi_f_rxkb_context_set_log_level, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 91), (void *)_cffi_d_rxkb_context_set_log_level },
  { "rxkb_context_set_user_data", (void *)_cffi_f_rxkb_context_set_user_data, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 101), (void *)_cffi_d_rxkb_context_set_user_data },
  { "rxkb_context_unref", (void *)_cffi_f_rxkb_context_unref, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 46), (void *)_cffi_d_rxkb_context_unref },
  { "rxkb_iso3166_code_get_code", (void *)_cffi_f_rxkb_iso3166_code_get_code, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 10), (void *)_cffi_d_rxkb_iso3166_code_get_code },
  { "rxkb_iso3166_code_next", (void *)_cffi_f_rxkb_iso3166_code_next, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 49), (void *)_cffi_d_rxkb_iso3166_code_next },
  { "rxkb_iso3166_code_ref", (void *)_cffi_f_rxkb_iso3166_code_ref, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 49), (void *)_cffi_d_rxkb_iso3166_code_ref },
  { "rxkb_iso3166_code_unref", (void *)_cffi_f_rxkb_iso3166_code_unref, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 49), (void *)_cffi_d_rxkb_iso3166_code_unref },
  { "rxkb_iso639_code_get_code", (void *)_cffi_f_rxkb_iso639_code_get_code, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 13), (void *)_cffi_d_rxkb_iso639_code_get_code },
  { "rxkb_iso639_code_next", (void *)_cffi_f_rxkb_iso639_code_next, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 55), (void *)_cffi_d_rxkb_iso639_code_next },
  { "rxkb_iso639_code_ref", (void *)_cffi_f_rxkb_iso639_code_ref, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 55), (void *)_cffi_d_rxkb_iso639_code_ref },
  { "rxkb_iso639_code_unref", (void *)_cffi_f_rxkb_iso639_code_unref, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 55), (void *)_cffi_d_rxkb_iso639_code_unref },
  { "rxkb_layout_first", (void *)_cffi_f_rxkb_layout_first, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 61), (void *)_cffi_d_rxkb_layout_first },
  { "rxkb_layout_get_brief", (void *)_cffi_f_rxkb_layout_get_brief, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 16), (void *)_cffi_d_rxkb_layout_get_brief },
  { "rxkb_layout_get_description", (void *)_cffi_f_rxkb_layout_get_description, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 16), (void *)_cffi_d_rxkb_layout_get_description },
  { "rxkb_layout_get_iso3166_first", (void *)_cffi_f_rxkb_layout_get_iso3166_first, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 52), (void *)_cffi_d_rxkb_layout_get_iso3166_first },
  { "rxkb_layout_get_iso639_first", (void *)_cffi_f_rxkb_layout_get_iso639_first, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 58), (void *)_cffi_d_rxkb_layout_get_iso639_first },
  { "rxkb_layout_get_name", (void *)_cffi_f_rxkb_layout_get_name, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 16), (void *)_cffi_d_rxkb_layout_get_name },
  { "rxkb_layout_get_popularity", (void *)_cffi_f_rxkb_layout_get_popularity, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 31), (void *)_cffi_d_rxkb_layout_get_popularity },
  { "rxkb_layout_get_variant", (void *)_cffi_f_rxkb_layout_get_variant, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 16), (void *)_cffi_d_rxkb_layout_get_variant },
  { "rxkb_layout_next", (void *)_cffi_f_rxkb_layout_next, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 64), (void *)_cffi_d_rxkb_layout_next },
  { "rxkb_layout_ref", (void *)_cffi_f_rxkb_layout_ref, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 64), (void *)_cffi_d_rxkb_layout_ref },
  { "rxkb_layout_unref", (void *)_cffi_f_rxkb_layout_unref, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 64), (void *)_cffi_d_rxkb_layout_unref },
  { "rxkb_model_first", (void *)_cffi_f_rxkb_model_first, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 67), (void *)_cffi_d_rxkb_model_first },
  { "rxkb_model_get_description", (void *)_cffi_f_rxkb_model_get_description, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 19), (void *)_cffi_d_rxkb_model_get_description },
  { "rxkb_model_get_name", (void *)_cffi_f_rxkb_model_get_name, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 19), (void *)_cffi_d_rxkb_model_get_name },
  { "rxkb_model_get_popularity", (void *)_cffi_f_rxkb_model_get_popularity, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 34), (void *)_cffi_d_rxkb_model_get_popularity },
  { "rxkb_model_get_vendor", (void *)_cffi_f_rxkb_model_get_vendor, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 19), (void *)_cffi_d_rxkb_model_get_vendor },
  { "rxkb_model_next", (void *)_cffi_f_rxkb_model_next, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 70), (void *)_cffi_d_rxkb_model_next },
  { "rxkb_model_ref", (void *)_cffi_f_rxkb_model_ref, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 70), (void *)_cffi_d_rxkb_model_ref },
  { "rxkb_model_unref", (void *)_cffi_f_rxkb_model_unref, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 70), (void *)_cffi_d_rxkb_model_unref },
  { "rxkb_option_first", (void *)_cffi_f_rxkb_option_first, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 76), (void *)_cffi_d_rxkb_option_first },
  { "rxkb_option_get_brief", (void *)_cffi_f_rxkb_option_get_brief, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 22), (void *)_cffi_d_rxkb_option_get_brief },
  { "rxkb_option_get_description", (void *)_cffi_f_rxkb_option_get_description, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 22), (void *)_cffi_d_rxkb_option_get_description },
  { "rxkb_option_get_name", (void *)_cffi_f_rxkb_option_get_name, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 22), (void *)_cffi_d_rxkb_option_get_name },
  { "rxkb_option_get_popularity", (void *)_cffi_f_rxkb_option_get_popularity, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 37), (void *)_cffi_d_rxkb_option_get_popularity },
  { "rxkb_option_group_allows_multiple", (void *)_cffi_f_rxkb_option_group_allows_multiple, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 7), (void *)_cffi_d_rxkb_option_group_allows_multiple },
  { "rxkb_option_group_first", (void *)_cffi_f_rxkb_option_group_first, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 79), (void *)_cffi_d_rxkb_option_group_first },
  { "rxkb_option_group_get_description", (void *)_cffi_f_rxkb_option_group_get_description, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 25), (void *)_cffi_d_rxkb_option_group_get_description },
  { "rxkb_option_group_get_name", (void *)_cffi_f_rxkb_option_group_get_name, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 25), (void *)_cffi_d_rxkb_option_group_get_name },
  { "rxkb_option_group_get_popularity", (void *)_cffi_f_rxkb_option_group_get_popularity, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 40), (void *)_cffi_d_rxkb_option_group_get_popularity },
  { "rxkb_option_group_next", (void *)_cffi_f_rxkb_option_group_next, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 82), (void *)_cffi_d_rxkb_option_group_next },
  { "rxkb_option_group_ref", (void *)_cffi_f_rxkb_option_group_ref, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 82), (void *)_cffi_d_rxkb_option_group_ref },
  { "rxkb_option_group_unref", (void *)_cffi_f_rxkb_option_group_unref, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 82), (void *)_cffi_d_rxkb_option_group_unref },
  { "rxkb_option_next", (void *)_cffi_f_rxkb_option_next, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 73), (void *)_cffi_d_rxkb_option_next },
  { "rxkb_option_ref", (void *)_cffi_f_rxkb_option_ref, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 73), (void *)_cffi_d_rxkb_option_ref },
  { "rxkb_option_unref", (void *)_cffi_f_rxkb_option_unref, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 73), (void *)_cffi_d_rxkb_option_unref },
};

static const struct _cffi_struct_union_s _cffi_struct_unions[] = {
  { "$FILE", 117, _CFFI_F_OPAQUE,
    (size_t)-1, -1, -1, 0 /* opaque */ },
  { "$va_list", 99, _CFFI_F_OPAQUE,
    (size_t)-1, -1, -1, 0 /* opaque */ },
  { "rxkb_context", 121, _CFFI_F_OPAQUE,
    (size_t)-1, -1, -1, 0 /* opaque */ },
  { "rxkb_iso3166_code", 122, _CFFI_F_OPAQUE,
    (size_t)-1, -1, -1, 0 /* opaque */ },
  { "rxkb_iso639_code", 123, _CFFI_F_OPAQUE,
    (size_t)-1, -1, -1, 0 /* opaque */ },
  { "rxkb_layout", 124, _CFFI_F_OPAQUE,
    (size_t)-1, -1, -1, 0 /* opaque */ },
  { "rxkb_model", 125, _CFFI_F_OPAQUE,
    (size_t)-1, -1, -1, 0 /* opaque */ },
  { "rxkb_option", 126, _CFFI_F_OPAQUE,
    (size_t)-1, -1, -1, 0 /* opaque */ },
  { "rxkb_option_group", 127, _CFFI_F_OPAQUE,
    (size_t)-1, -1, -1, 0 /* opaque */ },
};

static const struct _cffi_enum_s _cffi_enums[] = {
  { "rxkb_context_flags", 44, _cffi_prim_int(sizeof(enum rxkb_context_flags), ((enum rxkb_context_flags)-1) <= 0),
    "RXKB_CONTEXT_NO_FLAGS,RXKB_CONTEXT_NO_DEFAULT_INCLUDES,RXKB_CONTEXT_LOAD_EXOTIC_RULES" },
  { "rxkb_log_level", 93, _cffi_prim_int(sizeof(enum rxkb_log_level), ((enum rxkb_log_level)-1) <= 0),
    "RXKB_LOG_LEVEL_CRITICAL,RXKB_LOG_LEVEL_ERROR,RXKB_LOG_LEVEL_WARNING,RXKB_LOG_LEVEL_INFO,RXKB_LOG_LEVEL_DEBUG" },
  { "rxkb_popularity", 120, _cffi_prim_int(sizeof(enum rxkb_popularity), ((enum rxkb_popularity)-1) <= 0),
    "RXKB_POPULARITY_STANDARD,RXKB_POPULARITY_EXOTIC" },
};

static const struct _cffi_typename_s _cffi_typenames[] = {
  { "FILE", 117 },
  { "va_list", 99 },
};

static const struct _cffi_type_context_s _cffi_type_context = {
  _cffi_types,
  _cffi_globals,
  NULL,  /* no fields */
  _cffi_struct_unions,
  _cffi_enums,
  _cffi_typenames,
  68,  /* num_globals */
  9,  /* num_struct_unions */
  3,  /* num_enums */
  2,  /* num_typenames */
  NULL,  /* no includes */
  130,  /* num_types */
  1,  /* flags */
};

#ifdef __GNUC__
#  pragma GCC visibility push(default)  /* for -fvisibility= */
#endif

#ifdef PYPY_VERSION
PyMODINIT_FUNC
_cffi_pypyinit__ffi(const void *p[])
{
    if (((intptr_t)p[0]) >= 0x0A03) {
        _cffi_call_python_org = (void(*)(struct _cffi_externpy_s *, char *))p[1];
    }
    p[0] = (const void *)0x2601;
    p[1] = &_cffi_type_context;
#if PY_MAJOR_VERSION >= 3
    return NULL;
#endif
}
#  ifdef _MSC_VER
     PyMODINIT_FUNC
#  if PY_MAJOR_VERSION >= 3
     PyInit__ffi(void) { return NULL; }
#  else
     init_ffi(void) { }
#  endif
#  endif
#elif PY_MAJOR_VERSION >= 3
PyMODINIT_FUNC
PyInit__ffi(void)
{
  return _cffi_init("xkbregistry._ffi", 0x2601, &_cffi_type_context);
}
#else
PyMODINIT_FUNC
init_ffi(void)
{
  _cffi_init("xkbregistry._ffi", 0x2601, &_cffi_type_context);
}
#endif

#ifdef __GNUC__
#  pragma GCC visibility pop
#endif
